"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
#from libs import base
#from constants import *
#from utilities import connection_manager
from libs import base
from constants import *
from utilities import connection_manager
import Queue

"""
NOTE: Currently XDR is not secured. Once security is enabled for XDR port
pass on proper credentials to base.
"""
class Xdr(object):
    """Xdr contains all information about xdr"""

    def __new__(cls, ip, port, infoport, cluster_id, user=None, password=None):

        if base.is_xdr_enabled(ip, infoport) is False or infoport == port:
            return {}
        return super(Xdr, cls).__new__(cls, ip, port, infoport, cluster_id)

    def __init__(self, ip, port, infoport, cluster_id, user=None, password=None):

        self._ip = ip
        self._port = port
        self._infoport = infoport
        self._cluster_id = cluster_id
        self._statistics = {}
        self._modified_statistics = {}
        self._configuration = {}
        self._socket_queue = Queue.Queue(maxsize=SOCKET_QUEUE_SIZE)
        self._socket_queue = connection_manager.create_conn_queue(ip, infoport, int(XDR_SOCKET_QUEUE_SIZE), user=user, password=password)

        self._modified_configuration = {}

    def get_socket_from_queue(self):

        while self._socket_queue.empty():
            pass
        sock = self._socket_queue.get()
        #return self._socket_queue.get()
        return sock

    def put_socket_in_queue(self, sock):

        #import sys, inspect
        self._socket_queue.put(sock)


    def get_statistics(self, stats_type="all", user=None, password=None):

        sock = self.get_socket_from_queue()
        statistics = base.get_xdr_statistics(self._ip, self._infoport, sock= sock)


        #if statistics.get('node_status',None) is 'off':

            #new_sock = connection_manager.create_socket(self._ip. self._infoport, user=user, password=password)
            #if new_sock is not None:
              #  sock = new_sock
              #  statistics = base.get_xdr_statistics(self._ip, self._infoport, sock= sock)
        self.put_socket_in_queue(sock)
        self._modified_statistics = dict(set(statistics.items()) - set(self._statistics.items()))
        self._statistics = statistics
        return self._statistics if stats_type == "all" else self._modified_statistics

    def get_subset_of_statistics(self, attrs=[], user=None, password=None):

        subset = dict()

        stats = self.get_statistics(user=user, password=password)
        for attr in attrs:
            subset[attr] = stats.get(attr, NOT_SUPPORTED)
        status = stats.get('node_status')
        return subset, status

    def get_statistic(self, attr, user=None, password=None):

        stats = self.get_statistics(user=user, password=password)
        return stats.get(attr, NOT_SUPPORTED)

    def get_configurations(self, stats_type="all", user=None, password=None):

        sock = self.get_socket_from_queue()
        configuration = base.get_node_configuration(self._ip, self._infoport, sock=sock)
        #if configuration.get('node_status',None) is 'off':
            #new_sock = connection_manager.create_socket(self._ip. self._infoport, user=user, password=password)
            #if new_sock is not None:
             #   sock = new_sock
              #  configuration = base.get_node_configuration(self._ip, self._infoport, sock=sock)
        self.put_socket_in_queue(sock)
        self._modified_configuration = dict(set(configuration.items()) - set(self._configuration.items()))
        self._configuration = configuration
        return self._configuration if stats_type == "all" else self._modified_configuration

    def get_subset_of_configurations(self, attrs=[], user=None, password=None):

        subset = dict()
        configurations = self.get_configurations(user=user, password=password)

        for attr in attrs:
            subset[attr] = configurations.get(attr, NOT_SUPPORTED)
        return subset

    def get_configuration(self, attr, user=None, password=None):

        configurations = self.get_configurations(user=user, password=password)
        return configurations.get(attr, NOT_SUPPORTED)

    def update_configurations(self, user=None, password=None):

        sock = self.get_socket_from_queue()
        configuration = base.get_node_configuration(self._ip, self._infoport, clear_cache=True, sock=sock)
        #if configuration.get('node_status', None) is 'off':
            #new_sock = connection_manager.create_socket(self._ip. self._infoport, user=user, password=password)
            #if new_sock is not None:
            #    sock = new_sock
            #    configuration = base.get_node_configuration(self._ip, self._infoport, clear_cache=True, sock=sock)
        self.put_socket_in_queue(sock)
        self._modified_configuration = dict(set(configuration.items()) - set(self._configuration.items()))
        self._configuration = configuration

    def set_configuration(self, parameter, value, user=None, password=None):

        sock = self.get_socket_from_queue()
        status = base.set_xdr_configuration(self._ip, self._infoport, str(parameter), str(value), sock=sock)
        #if status is False:
         #   new_sock = connection_manager.create_socket(self._ip,self._infoport, user=user, password=password)
          #  if new_sock is not None:
           #     sock = new_sock
            #    status = base.set_xdr_configuration(self._ip, self._infoport, str(parameter), str(value), sock=sock)
        self.put_socket_in_queue(sock)
        return status

    def set_configurations(self, edit_parameters, user=None, password=None):

        error_list = list()
        for key, value in list(edit_parameters.items()):
            if self.set_configuration(key, value, user=user, password=password) is False:
                error_list.append(key)
        self.update_configurations(user=user, password=password)

        return error_list

    def get_dc_statistics(self, dc, user=None, password=None):

        #sock = self.get_socket_from_queue()
        #self._dc_statistics = base.get_dc_statistics(self._ip, self._infoport, dc, sock=sock)
        self._dc_statistics = base.get_dc_statistics(self._ip, self._infoport, dc)
        """if self._dc_statistics is False:
            new_sock = connection_manager.create_socket(self._ip, self._infoport, user=user, password=password)
            if new_sock is not None:
                sock = new_sock
                self._dc_statistics = base.get_dc_statistics(self._ip, self._infoport, dc, sock=sock)"""
        #self.put_socket_in_queue(sock)
        return self._dc_statistics

    def get_subset_of_dc_statistics(self, attrs=[], user=None, password=None):

        """
        This method currently not in USE
        """
        subset = dict()

        stats = self.get_dc_statistics(user=user, password=password)
        for attr in attrs:
            subset[attr] = stats.get(attr, NOT_SUPPORTED)
        return subset

    def get_dc_statistic(self, attr, user=None, password=None):

        """
        This method currently not in USE
        """
        stats = self.get_dc_statistics(user=user, password=password)
        return stats.get(attr, NOT_SUPPORTED)

    def get_dc_names(self, user=None, password=None):

        sock = self.get_socket_from_queue()
        names = base.get_dc_names(self._ip, self._infoport, sock=sock)
        #if type(names) is int:
         #   new_sock = connection_manager.create_socket(self._ip, self._infoport, user=user, password=password)
          #  if new_Sock is not None:
            #    sock = new_sock
           #     names = base.get_dc_names(self._ip, self._infoport, sock=sock)
        self.put_socket_in_queue(sock)
        return names

    def get_xdr_port(self):

        return self._infoport

    def replace_socket_queue(self, user, password):

        self._socket_queue = connection_manager.create_conn_queue(self._ip, self._infoport, int(XDR_SOCKET_QUEUE_SIZE), user=user, password=password)

    def get_socket_queue(self):

        return self._socket_queue